#ifndef SENSOR_H
#define SENSOR_H

/**
 * @brief Sensor Component
 *
 * Simulates a sensor reading by generating a random number.
 *
 * @return A random double between 0 and 1.
 */
double read_sensor();

#endif
